/*************************************************
* Zeus CMD - OpenGL ES Tutorial 21 : Waving Flag *
* By Grant James (ZEUS)                          *
* http://www.zeuscmd.com                         *
*************************************************/
#pragma comment(lib, "libGLES_CM.lib")

#define GLUTES_STATIC

#include <GLES/glutes.h>
#include <GLES/glues.h>
#include <GLES/glues.c>

#define PIOVER180 0.01745329252

GLuint texture[1];
float flagPoints[10][36][3];
float flagCoords[10][36][2];
float wrapValue;

bool wireFrame = false;

float xrot = 0.0f;
float yrot = 0.0f;

void initFlag()
{
	int x;
	int y;
	int apos;

	for (y = 0; y < 10; y++)
	{
		for (x = 0, apos = 0; x < 18; x++, apos += 2)
		{
			flagPoints[y][apos][0] = float(x);
			flagPoints[y][apos][1] = float(y + 1);
			flagPoints[y][apos][2] = float(sin(x * 20.0f * PIOVER180)) * 1.5f;

			flagCoords[y][apos][0] = x * 1.0f / 18.0f;
			flagCoords[y][apos][1] = (y + 1) * 1.0f / 10.0f;

			flagPoints[y][apos + 1][0] = float(x);
			flagPoints[y][apos + 1][1] = float(y);
			flagPoints[y][apos + 1][2] = float(sin(x * 20.0f * PIOVER180)) * 1.5f;

			flagCoords[y][apos + 1][0] = x * 1.0f / 18.0f;
			flagCoords[y][apos + 1][1] = y * 1.0f / 10.0f;
		}
	}
}

unsigned char *loadBMP(char *filename, BITMAPINFOHEADER *bmpInfo)
{
	FILE *file;
	BITMAPFILEHEADER bmpFile;
	unsigned char *bmpImage = NULL;
	unsigned char tmpRGB;

	TCHAR path[256];
	char fullPath[256];
	GetModuleFileName(NULL, path, 256);

	TCHAR *pos = wcsrchr(path, '\\');
	*(pos + 1) = '\0';
	
	wcstombs(fullPath, path, 256);
	
	strcat(fullPath, filename);

	file = fopen(fullPath,"rb");

	if (!file)
	{
		MessageBox(NULL, L"Can't Find Bitmap", L"Error", MB_OK);
		return NULL;
	}

	fread(&bmpFile,sizeof(BITMAPFILEHEADER),1,file);

	if (bmpFile.bfType != 0x4D42)
	{
		MessageBox(NULL, L"Incorrect texture type", L"Error", MB_OK);
		fclose(file);
		return NULL;
	}

	fread(bmpInfo,sizeof(BITMAPINFOHEADER),1,file);

	fseek(file,bmpFile.bfOffBits,SEEK_SET);

	bmpImage = new unsigned char[bmpInfo->biSizeImage];

	if (!bmpImage)
	{
		MessageBox(NULL, L"Out of Memory", L"Error", MB_OK);
		delete[] bmpImage;
		fclose(file);
		return NULL;
	}

	fread(bmpImage,1,bmpInfo->biSizeImage,file);

	if (!bmpImage)
	{
		MessageBox(NULL, L"Error reading bitmap", L"Error", MB_OK);
		fclose(file);
		return NULL;
	}

	for (unsigned int i = 0; i < bmpInfo->biSizeImage; i+=3)
	{
		tmpRGB = bmpImage[i];
		bmpImage[i] = bmpImage[i+2];
		bmpImage[i+2] = tmpRGB;
	}

	fclose(file);

	return bmpImage;
}

bool loadTextures()
{
	BITMAPINFOHEADER info;
	unsigned char *bitmap = NULL;
	
	bitmap = loadBMP("saflag.bmp", &info);

	if (!bitmap)
		return false;

	glGenTextures(1, texture);

	glBindTexture(GL_TEXTURE_2D, texture[0]);

	glTexParameterf(GL_TEXTURE_2D, 
		GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D,
		GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, info.biWidth,
		info.biHeight, 0, GL_RGB, GL_UNSIGNED_BYTE,
		bitmap);

	delete[] bitmap;

	return true;
}

bool init()
{
	if (!loadTextures())
	{
		MessageBox(NULL, L"Error loading textures", L"Error", MB_OK);
		return false;
	}

	initFlag();

	glEnable(GL_TEXTURE_2D);

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepthf(1.0f);

	glVertexPointer(3, GL_FLOAT, 0, flagPoints);
	glTexCoordPointer(2, GL_FLOAT, 0, flagCoords);
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	glShadeModel(GL_SMOOTH);

	return true;
}

void display()
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	glTranslatef(-3.0f,-3.0f,-60.0f);

	glPushMatrix();

	glRotatef(xrot,1.0f,0.0f,0.0f);
	glRotatef(yrot,0.0f,1.0f,0.0f);

	for (int i = 0; i < 10; i++)
		glDrawArrays(wireFrame ? 
			GL_LINE_STRIP : GL_TRIANGLE_STRIP, i * 36, 36);

	glFlush ();
	glutSwapBuffers();
}

void idle()
{
	xrot += 2.0f;
	yrot += 3.0f;

	wrapValue = flagPoints[0][35][2];

	for (int y = 0; y < 10; y++)
	{
		for (int x = 35; x > 0; x -= 2)
		{
			flagPoints[y][x][2] = flagPoints[y][x-2][2];
			flagPoints[y][x-1][2] = flagPoints[y][x-3][2];
		}

		flagPoints[y][0][2] = wrapValue;
		flagPoints[y][1][2] = wrapValue;
	}

	glutPostRedisplay();
}

void keyboard2(int key, int x, int y)
{
	switch(key)
	{
	case GLUT_KEY_UP : exit(0); break;
	}
}

void reshape(int width, int height)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glViewport(0, 0, width, height);
	gluPerspectivef(45.0f, 1.0f * width / height, 1.0f, 100.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void menu(int entry)
{
	switch(entry)
	{
	case 1 : exit(0); break;
	case 2 :
		wireFrame = !wireFrame;
		break;
	}
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);

	glutCreateWindow("21 - Waving Flag");

	if (!init())
		return 1;

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutSpecialFunc(keyboard2);

	glutIdleFunc(idle);

	glutCreateMenu(menu);
	glutAddMenuEntry("Toggle Wireframe", 2);
	glutAddMenuEntry("Quit", 1);

	glutAttachMenu(GLUT_LEFT_BUTTON);
	
	glutMainLoop();

	return 0;
}
